# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_csv', 'django_csv.migrations']

package_data = \
{'': ['*']}

install_requires = \
['django>=2.2,<4.0']

setup_kwargs = {
    'name': 'django-csv-downloads',
    'version': '0.1.0',
    'description': 'Django app for tracking queryset-backed CSV downloads',
    'long_description': '# Django CSV Downloads\n\nDjango app for tracking queryset-backed CSV downloads\n',
    'author': 'YunoJuno',
    'author_email': 'code@yunojuno.com',
    'maintainer': 'YunoJuno',
    'maintainer_email': 'code@yunojuno.com',
    'url': 'https://github.com/yunojuno/django-csv-downloads',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from django.db.models.query import QuerySet
from django.http import HttpResponse
from django.http.request import HttpRequest

from .csv import write_csv
from .models import CsvDownload


def download_csv(
    request: HttpRequest, filename: str, queryset: QuerySet, *columns: str
) -> HttpResponse:
    """
    Download queryset as a CSV.

    This function creates the HttpResponse and tracks the download against
    the request.real_user, giving an audit trail of downloads.

    """
    response = HttpResponse(content_type="text/csv")
    response["Content-Disposition"] = f'attachment; filename="{filename}"'
    row_count = write_csv(response, queryset, *columns)
    response["X-Row-Count"] = row_count
    CsvDownload.objects.create(
        user=request.user,
        row_count=row_count,
        filename=filename,
        columns=", ".join(columns),
    )
    return response



"""
Functions used to write QuerySet data to a CSV.

This module contains a class (QuerySetWriter) used to wrap up writing of
data from a QuerySet to a CSV (file or HttpResponse).

You should never need to call the QuerySetWriter explicitly - but instead
use the `write_csv` function.

Example of writing to a file:

    >>> qs = MyModel.objects.all()
    >>> cols = ("col1", "col2", "col3")
    >>> with open('foo.csv', 'w') as csvfile:
    >>>     csv.write_csv(csvfile, qs, *cols)
    10

Example of writing to an HttpResponse:

    >>> response = HttpResponse(content_type="text/csv")
    >>> response["Content-Disposition"] = 'attachment; filename="foo.csv"'
    >>> csv.write_csv(response, qs, *cols)
    10

Example of writing to an in-memory text buffer:

    >>> buffer = io.StringIO()
    >>> csv.write_csv(buffer, qs, *cols)
    10

"""
import csv
from typing import Any

from django.db.models import QuerySet

from .settings import MAX_ROWS


class QuerySetWriter:
    """
    Class used to write contents of a QuerySet to a CSV.

    This class wraps the csv.writerow and csv.writerows functions, and
    maps queryset columns to CSV fields (via `values_list`). It is
    initialised with a 'csvfile' where "csvfile can be any object with
    a write() method."

    See https://docs.python.org/3/library/csv.html#csv.writer

    """

    def __init__(
        self,
        csvfile: Any,
        queryset: QuerySet,
        *columns: str,
    ) -> None:
        self._writer = csv.writer(csvfile)
        self.columns = columns
        self.rows = queryset.values_list(*columns)
        self.row_count = 0

    def write_header(self) -> None:
        self._writer.writerow(self.columns)

    def write_rows(self, max_rows: int = MAX_ROWS) -> None:
        """Write QuerySet contents out to CSV and set row_count."""
        self._writer.writerows(rows := self.rows[:max_rows])
        self.row_count = rows.count()


def write_csv(csvfile: Any, queryset: QuerySet, *columns: str) -> int:
    """Write QuerySet to fileobj in CSV format."""
    writer = QuerySetWriter(csvfile, queryset, *columns)
    writer.write_header()
    writer.write_rows()
    return writer.row_count
